package javax.swing.plaf.basic;

import java.awt.dnd.DragSource;
import sun.awt.dnd.SunDragSourceContextPeer;
import javax.swing.SwingUtilities;
import javax.swing.TransferHandler;
import sun.awt.AppContext;
import javax.swing.JComponent;
import java.awt.event.MouseEvent;

class DragRecognitionSupport
{
    private int motionThreshold;
    private MouseEvent dndArmedEvent;
    private JComponent component;
    
    DragRecognitionSupport() {
    }
    
    private static DragRecognitionSupport getDragRecognitionSupport() {
        DragRecognitionSupport value = (DragRecognitionSupport)AppContext.getAppContext().get(DragRecognitionSupport.class);
        if (value == null) {
            value = new DragRecognitionSupport();
            AppContext.getAppContext().put(DragRecognitionSupport.class, value);
            return value;
        }
        return value;
    }
    
    public static boolean mousePressed(MouseEvent mouseEvent) {
        return getDragRecognitionSupport().mousePressedImpl(mouseEvent);
    }
    
    public static MouseEvent mouseReleased(MouseEvent mouseEvent) {
        return getDragRecognitionSupport().mouseReleasedImpl(mouseEvent);
    }
    
    public static boolean mouseDragged(MouseEvent mouseEvent, DragRecognitionSupport.BeforeDrag beforeDrag) {
        return getDragRecognitionSupport().mouseDraggedImpl(mouseEvent, beforeDrag);
    }
    
    private void clearState() {
        this.dndArmedEvent = null;
        this.component = null;
    }
    
    private int mapDragOperationFromModifiers(MouseEvent anEvent, TransferHandler transferHandler) {
        if (transferHandler == null || !SwingUtilities.isLeftMouseButton(anEvent)) {
            return 0;
        }
        return SunDragSourceContextPeer.convertModifiersToDropAction(anEvent.getModifiersEx(), transferHandler.getSourceActions(this.component));
    }
    
    private boolean mousePressedImpl(MouseEvent dndArmedEvent) {
        this.component = (JComponent)dndArmedEvent.getSource();
        if (this.mapDragOperationFromModifiers(dndArmedEvent, this.component.getTransferHandler()) != 0) {
            this.motionThreshold = DragSource.getDragThreshold();
            this.dndArmedEvent = dndArmedEvent;
            return true;
        }
        this.clearState();
        return false;
    }
    
    private MouseEvent mouseReleasedImpl(MouseEvent mouseEvent) {
        if (this.dndArmedEvent == null) {
            return null;
        }
        MouseEvent dndArmedEvent = null;
        if (mouseEvent.getSource() == this.component) {
            dndArmedEvent = this.dndArmedEvent;
        }
        this.clearState();
        return dndArmedEvent;
    }
    
    private boolean mouseDraggedImpl(MouseEvent mouseEvent, DragRecognitionSupport.BeforeDrag beforeDrag) {
        if (this.dndArmedEvent == null) {
            return false;
        }
        if (mouseEvent.getSource() != this.component) {
            this.clearState();
            return false;
        }
        int abs = Math.abs(mouseEvent.getX() - this.dndArmedEvent.getX());
        int abs2 = Math.abs(mouseEvent.getY() - this.dndArmedEvent.getY());
        if (abs > this.motionThreshold || abs2 > this.motionThreshold) {
            TransferHandler transferHandler = this.component.getTransferHandler();
            int mapDragOperationFromModifiers = this.mapDragOperationFromModifiers(mouseEvent, transferHandler);
            if (mapDragOperationFromModifiers != 0) {
                if (beforeDrag != null) {
                    beforeDrag.dragStarting(this.dndArmedEvent);
                }
                transferHandler.exportAsDrag(this.component, this.dndArmedEvent, mapDragOperationFromModifiers);
                this.clearState();
            }
        }
        return true;
    }
    
    public interface BeforeDrag
    {
        public abstract void dragStarting(MouseEvent p0);
    }
}
